var Event = require('./Event.js').Event;
var Class = require("../class/Class.js").Class;

/**
 * EventDispatcher
 * @class EventDispatcher provides listening and dispatching events functionality
 * @requires Class
 * @requires Event
 */
var EventDispatcher = function() {

    /**
     * @property {Object} _eventsArray
     * @private
     */
    this._eventsArray = new Object();

    /**
     * Checks if given event type listener is registered
     * @param {String} type Event type
     * @returns {Boolean} True if listener of given type is registered 
     */
    this.hasEventListener = function(type) {
        if (this._eventsArray.hasOwnProperty(type) && this._eventsArray[type].length > 0){
            return true;
        }
        return false;
    };

    /**
     * Adding listener
     * @param {String} type Event type
     * @param {Function} callbackFunction Function that will be called if event of givent type will ocurr. 
     */
    this.addEventListener = function(type, callbackFunction, context){
        //creating new listener object
        var listener = new Object();
        listener.callbackFunction = callbackFunction;
        listener.context = (context) ? context : this;
        //creating space for listeners of given type
        if (!this.hasEventListener(type)){
            this._eventsArray[type] = new Array();
        }
        
        //check if given listener is arledy added.
        for(var i = 0, l = this._eventsArray[type].length; i < l; i++){
            if(this._eventsArray[type][i].callbackFunction == callbackFunction && this._eventsArray[type][i].context == context){
                return false;
            }
        }
        
        //adding listener
        this._eventsArray[type].push(listener);

        return this;
    };

    /**
     * Dispatches event
     * @param {Event} e Event object to be dispatched
     */
    this.dispatchEvent = function(e) {
        if (this.hasEventListener(e.type)) {
            //creating private function to pass listener by value otherwise nextTick oparates on last loop iteration variables states 
            var that = this;
            var callLater = function(listener) {                
                //passing script execution to end of script execution queue
                //different in client "window.setTimeout"
                //process.nextTick( function(){
                //setTimeout( function(){ 
                    listener.callbackFunction.call(listener.context, e);
                //}, 0);
            };
            //creating snapshot of current listener's list  
            var snapshot = this._eventsArray[e.type];
            //dispatching events
            for (var i = 0; i < snapshot.length; i++) {
                e.target = this;
                var listener = snapshot[i];
                if ( listener.callbackFunction != undefined ){
                    callLater(listener);
                }
                else{
                    throw "Callback function for Event type: [ " + e.type + " ] is undefined. Please check your addEventListener statement.";
                }
            }
            //deleting snapshot
            delete snapshot;
        }
    };

    /**
     * Remove listener
     * @param {String} type Event type
     * @param {Function} callbackFunction Function that was given on adding event listener. 
     */
    this.removeEventListener = function(type, callbackFunction) {
        var i = 0;
        if (!this.hasEventListener(type)){
            throw 'Listeners of given type: "' + type + '" do not exists.';
        }             
        while(i < this._eventsArray[type].length && this._eventsArray[type][i].callbackFunction != callbackFunction) {
            i++;
        }
        if(i < this._eventsArray[type].length){
            if(this._eventsArray[type].length > 1){
                this._eventsArray[type].splice(i, 1);
            }
            else{
                delete this._eventsArray[type];
            }
            return true;
        }
        return false;
    };

    /**
     * Remove group of listeners or all listeners
     * @param {String} type Event type if given removes all listeners of given type, otherwise removes all listeners. 
     */
    this.removeAllEventListeners = function(type) { 
        if(type != undefined){
            if(this.hasEventListener(type)){
                delete this._eventsArray[type];
                return true;
            }
            else{
                throw 'Listeners of given type: "' + type + '" do not exists.';
            }
        }
        else{
            delete this._eventsArray;
            this._eventsArray = new Object();
            return true;
        }
    };
};

EventDispatcher = new Class(new EventDispatcher());
exports.EventDispatcher = EventDispatcher;
