var Class = require('../class/Class.js').Class;
var JsonRpcAbstract = require('./JsonRpcAbstract.js').JsonRpcAbstract;

/**
 * @class JsonRpcError
 * @extends JsonRpcAbstract
 * @requires Class
 * @requires JsonRpcAbstract
 *
 * @param {Number} code Number error code
 * @param {String} message error message
 * @param {Object} data additional error data
 */
var JsonRpcError = function(){

    /** @ignore */
    this.Extends = JsonRpcAbstract;

    /**
     * error code
     * 
     * @private
     * @property {Number} _code
     */
    this._code;

    /**
     * error message
     * 
     * @private
     * @property {String} _message
     */
    this._message;

    /**
     * optional additional error data
     * 
     * @private
     * @property {Object} _data
     */
    this._data;

    /** @ignore */
    this.initialize = function(/*data|code, message, data*/){
		this.parent.apply(this, arguments);
    };

	this._init = function(args){

		if(args[0]){
			this.setCode(args[0]);
		}else{
			throw JsonRpcAbstract.Exceptions.INVALID_JSON_RPC;
		}

		if(args[1] && (typeof args[1] ==='string')){
	        this.setMessage(args[1]);
		}else{
			throw JsonRpcAbstract.Exceptions.INVALID_JSON_RPC;
		}

		if(args[2]){
	        this.setData(args[2]);
		}
	};

	this._parse = function(json){
		return [
			json.code,
			json.message,
			json.data
		];
	};
    
    /**
     * sets error code
     * 
     * @param {Number} code error code
     */
    this.setCode = function(code){
        if(!code || code.constructor !== Number){
            throw "Code must be a Number!";
        }
        this._code = code;

        return this;
    };
    
    /**
     * gets error code
     * 
     * @returns {Number}
     */
    this.getCode = function(){
        return this._code;
    };
    
    /**
     * sets error message
     * 
     * @param {String} message error message
     * @returns {JsonRpcError}
     */
    this.setMessage = function(message){
        if(!message || message.constructor !== String){
            throw "Message must be a String!";
        }

        this._message = message;

        return this;
    };
    
    /**
     * gets error message
     * 
     * @returns {String}
     */
    this.getMessage = function(){
        return this._message;
    };
    
    /**
     * sets error data
     * 
     * @param {Object} data error data
     */
    this.setData = function(data){
        if(data instanceof JsonRpcError){
            this._data = data.toJson();
        }else{
            this._data = data;            
        }

        return this;
    };
    
    /**
     * gets error data
     * 
     * @returns {Object}
     */
    this.getData = function(){
        return this._data;
    };
    
    /**
     * returns properties as json
     * 
     * @returns {Object}
     */
    this.toJson = function(){
        var data = {
            code: this._code,
            message: this._message
        };

        if(this._data){
            data.data = this._data;
        }

        return data;
    };

};

JsonRpcError = new Class(new JsonRpcError());

/**
 * creates new instance based on the given instance
 * factory method
 * 
 * @static
 * 
 * @param {JsonRpcError} jsonRpcError
 * @returns {JsonRpcError}
 */
JsonRpcError.factory = function(jsonRpcError){
    if(!(jsonRpcError instanceof JsonRpcError)){
        throw JsonRpcError.Exceptions.WRONG_CLASS;
    }
    
    var factory = new JsonRpcError(jsonRpcError.getCode(), jsonRpcError.getMessage());

    return factory;
};

/**
 * @static
 * @constant
 * @namespace
 */
JsonRpcError.Exceptions = {};
/**
 * @static
 * @constant
 */
JsonRpcError.Exceptions.WRONG_CLASS = 'Given something is not a instance of JsonRpcError class';

exports.JsonRpcError = JsonRpcError;
