var Class = require("../class/Class.js").Class;
var JsonRpcAbstract = require("./JsonRpcAbstract.js").JsonRpcAbstract;

/**
 * @class JsonRpcRequest
 * @extends JsonRpcAbstract
 * @requires Class
 * @requires JsonRpcAbstract
 *
 * @param {Object} | {String} configuration
 */
var JsonRpcRequest = function(){

    /** @ignore */
    this.Extends = JsonRpcAbstract;

    /**
     * @property {Number} _id holds json rpc id property
     * @private
     */
    this._id = null;

    /**
     * @property {String} _method holds json rpc method name
     * @private
     */
    this._method = null;

    
    /**
     * @property {Object} _params holds json rpc parameters
     * @private
     */
    this._params = {};

   
    /** @ignore */
    this.initialize = function(/*data|id, method, params*/){
		this.parent.apply(this, arguments);
    };

	this._init = function(args){
		if(args[0]){
			this.setId(args[0]);
		}else{
			throw JsonRpcAbstract.Exceptions.INVALID_JSON_RPC;
		}

		if(args[1]){
			this.setMethod(args[1]);
		}else{
			throw JsonRpcAbstract.Exceptions.INVALID_JSON_RPC;
		}

		if(args[2]){
			this.setParams(args[2]);
		}
	};

	this._parse = function(json){
		return [
			json.id,
			json.method,
			json.params
		]; 
	};

	this.setId = function(id){

        if(typeof id !== 'string'){
            throw "Id should be a string";
        }
        
		this._id = id;

		return this;
	};

    /**
     * returns json rpc id
     *
     * @returns {Number}
     */
    this.getId = function(){
        return this._id;
    };

	this.setMethod = function(method){

        if(typeof method !== 'string'){
            throw "Method should be a string";
        }

		this._method = method;

		return this;
	};
    /**
     * returns json rpc method parameter
     *
     * @returns {String}
     */
    this.getMethod = function(){
        return this._method;
    };

	this.setVersion = function(version){
        if(typeof version !== 'string'){
            throw "Version should be a string";
        }
		this._jsonrpc = version;
		
		return this;
	}
    /**
     * returns json rpc version number
     *
     * @returns {Number}
     */
    this.getVersion = function(){
        return this._jsonrpc;
    };

	this.setParams = function(params){
		this._params = params;

		return this;
	};
    /**
     * returns json rpc params data
     *
     * @returns {Object}
     */
    this.getParams = function(){
        return this._params;
    };

	this.setParam = function(key, value){
        if(typeof key !== 'string'){
            throw "Key should be a string";
        }

		this._params[key] = value;

		return this;
	};
    /**
     * returns single json rpc param
     *
     * @param {String} key
     * @returns {Object}
     */
    this.getParam = function(key){
        if(this._params.hasOwnProperty(key)){
            return this._params[key];
        }
        return null;
    };

    /**
     * returns properties as json
     *
     * @returns {Object}
     */
    this.toJson = function(){
        return {
            "jsonrpc": this._jsonrpc,
			"id": this._id,
			"method": this._method,
			"params": this._params
        };
    };

	this.toString = function(){
		return JSON.stringify(this.toJson());
	};
};

JsonRpcRequest = new Class(new JsonRpcRequest());
exports.JsonRpcRequest = JsonRpcRequest;
