var http = require('http');
var Request = require('./Request.js').Request;
var Response = require('./Response.js').Response;
var Event = require('../event/Event.js').Event;
var ErrorEvent = require('../event/ErrorEvent.js').ErrorEvent;
var EventDispatcher = require('../event/EventDispatcher.js').EventDispatcher; 
var Class = require('../class/Class.js').Class;

var Loader = function(){

    /** @ignore */
    this.Extends = EventDispatcher;

    /**
     * @private
     * @property {Object} _buffer
     */
    this._buffer = null;

     /**
     * @private
     * @property {Object} _request
     */
    this._request = null;

     /**
     * @private
     * @property {Number} _timeout
     */
    this._timeout = 15000;

     /**
     * @private
     * @property {Object} _timer
     */
    this._timer = null;

	 /**
	 * @private
	 * @property {Boolean} _followRedirects
	 */
	this._followRedirects = true;

	 /**
	 * @private
	 * @property {Number} _maxRedirectsCount
	 */
	this._maxRedirectsCount = 10;

	 /**
	 * @private
	 * @property {Number} _currentRedirectsCount
	 */
	this._currentRedirectsCount = 0;

    /** @ignore */
    this.initialize = function(request){
        this.setRequest(request);
    };

    /**
     * callback responsible for gathering response chunks
     * @private
     * @param {Server.HttpResponse} response
     */
    this._response = function(response){
        var self = this;

		var dataLength = 0, buffers = [];

        response.addListener('data', function(buffer){
			buffers.push(buffer);
			dataLength += buffer.length;
        }).addListener('end', function(){
			self._buffer = new Buffer(dataLength), bufferPosition = 0;

			for(var i = 0, l = buffers.length; i < l; i++){
				buffers[i].copy(self._buffer, bufferPosition);
				bufferPosition += buffers[i].length;
			}

            self._onEnd(response);
        });
    };

    /**
     * sets timer who is responsible for timeout detection
     * @private
     * @param {Server.HttpRequest} request
     */
    this._setTimer = function(request){
        var self = this;
        this._timer = setTimeout(function(){
            request.removeAllListeners('response');
            self.dispatchEvent(new ErrorEvent(Loader.Event.ERROR, null, -3, 'Http response timeout'));
        }, this.getTimeout());
    };

    /**
     * clears timer handle that is responsible for timeout detection
     * @private
     */
    this._clearTimer = function(){
        clearTimeout(this._timer);
    };

	this._clearChunks = function(){
		this._buffer = [];
	};

    /**
     * called on request end, when whole data is retreived from server
     * @private
     */
    this._onEnd = function(response){
		var requestMethod = this._request.getMethod();
		if(response.statusCode >= 300 && response.statusCode < 400 && response.headers.hasOwnProperty('location') && this._followRedirects && requestMethod != Request.PUT && requestMethod != Request.POST){
			if(this._currentRedirectsCount >= this._maxRedirectsCount){
				this.dispatchEvent(new ErrorEvent(Loader.Event.ERROR, null,	-4, 'maxRedirectsCount reached, stopping'));
				return;
			}

			this._currentRedirectsCount++;
			this._request.setUrl(response.headers.location);
			this._clearChunks();
			this.load();
			return;
		}

		var data = this._parseResponse(response);

		if(data == false){
			//tutaj zakladamy ze _parseResponse obsluzyl blad
			return;
		}

        this.dispatchEvent(new Event(Loader.Event.LOADED, data));
    };

	this._parseResponse = function(response){
		var r = new Response({
			"statusCode": response.statusCode,
			"headers": response.headers,
			"httpVersion": response.httpVersion,
			"body": this._buffer
		});

		return r;
	};

	this._createRequest = function(request){
        
		var host = request.getHost();
		var port = request.getPort();
		var path = request.getPath();
		var method = request.getMethod();
		var headers = request.getHeaders();
		
		if(!host || !port || !path || !method || !headers){
			throw Loader.Exception.INCOMPLETE_REQUEST;
		}

		return {
			host: host,
			port: port,
			path: path + request.getQueryString(),
			method: method,
			headers: headers
		};
	};

    /**
     * sets timeout in miliseconds
     * @param {Number} ms
     */
    this.setTimeout = function(ms){
        this._timeout = ms;
        return this;
    };

    /**
     * gets timeout in miliseconds
     * @returns {Number}
     */
    this.getTimeout = function(){
        return this._timeout;
    };

	this.setFollowRedirects = function(follow){
		if(typeof follow == 'boolean'){
			this._followRedirects = follow;
		}else{
			throw Loader.Exception.FOLLOW_REDIRECTS_TYPE_ERROR;
		}
		return this;
	};

	this.getFollowRedirects = function(){
		return this._followRedirects;
	};

	this.setMaxRedirectsCount = function(count){
		this._maxRedirectsCount = count;

		return this;
	};

	this.getMaxRedirectsCount = function(){
		return this._maxRedirectsCount;
	};

    /**
     * sets request object to process
     * @param {OpalRequest} request
     */
    this.setRequest = function(request){
        if(request && !(request instanceof Request)){
            throw Loader.Exception.WRONG_REQUEST_INSTANCE;
        } else if(request) {
            this._request = new Request(request);
			this._currentRedirectCount = 0;

			this._clearChunks();
        }        
    };

    /**
     * makes request to the backend system based on the request object
     * @param {Request} request
     */
    this.load = function(request){
        this.setRequest(request);
        
        if(this._request == null){
            throw Loader.Exception.REQUEST_NOT_SET;
        }

        var self = this;
        
        var request = http.request(this._createRequest(this._request), function(response){
			self._clearTimer();
            self._response(response);
        }).addListener('error', function(e){
            self._clearTimer();
            self.dispatchEvent(new ErrorEvent(Loader.Event.ERROR, e, -1, 'Http request error'));
        });

        request.write(this._request.getBody().toBuffer());
        request.end();

        this._setTimer(request);
    };

};

Loader = new Class(new Loader());

Loader.Event = {};

Loader.Event.LOADED = "Loader_LOADED";
Loader.Event.ERROR = "Loader_ERROR";

Loader.Exception = {};
Loader.Exception.INCOMPLETE_REQUEST = 'Given request misses some key values';
Loader.Exception.REQUEST_NOT_SET = 'Request has not been set';
Loader.Exception.WRONG_REQUEST_INSTANCE = 'Given request is not a instance of Request';
Loader.Exception.FOLLOW_REDIRECTS_TYPE_ERROR = 'setFollowRedirects requires an argument to be a boolean';
exports.Loader = Loader;
