var Class = require('../class/Class.js').Class;
var Assertions = require('../common/Assertions.js');
var Types = require('../common/Types.js');
var BinaryData = require('../data/BinaryData.js').BinaryData;
var Iconv = (require('os').type().search('cygwin') !== -1) ? require('iconv/iconv_cygwin').Iconv : require('iconv').Iconv;

/**
 * @class Message
 * @requires Class
 *
 * @param {String} url
 */
var Message = function(){

    /**
     * @private
     * @property {Object} _headers
     */
    this._headers = {};

    /**
     * @private
     * @property {Object} _body
     */
    this._body = '';
   
	this._httpVersion = '1.0';

	this._cookieKey = null;

    /** @ignore */
    this.initialize = function(params){
		if(Buffer.isBuffer(params)){
			this.init(this._parseBuffer(params));
		}else{
			this.init(params);
		}
    };

	this.init = function(params){
		if(params && typeof params == 'object' && params.constructor == Object){
			var str, methodName;
			for(var key in params){
				if(params.hasOwnProperty(key)){
					methodName = key.charAt(0).toUpperCase() + key.substr(1);
					if(typeof this['set' + methodName] == 'function'){
						if(typeof this['_set' + methodName] == 'function'){
							this['_set' + methodName](params[key]);
						}else{
                            this['set' + methodName](params[key]);
                        }
					}
				}
			}
		}
	};

	this._parseBuffer = function(buffer){
		var headersEnd = -1;
		var bodyStart = -1;

		for(var i = 0, l = buffer.length; i < l; i++){
			if(buffer[i] == 13 && buffer[i+2] == 13 && buffer[i+1] == 10 && buffer[i+3] == 10){
				headersEnd = i;
				bodyStart = i+4;
				break;
			}
		}

		Assertions.equal(true, headersEnd > -1, Message.Exception.WRONG_BUFFER);
		Assertions.equal(true, bodyStart > -1, Message.Exception.WRONG_BUFFER);
			
		var resp = this._parseHead(buffer.slice(0, headersEnd).toString('utf8'));
		resp.body = buffer.slice(bodyStart);

		return resp;
	};

	this._parseHead = function(data){
		var headers = data.split("\r\n"), statusLine = this._parseStatusLine(headers.shift());

		return {
			headers: this._parseHeaders(headers),
			statusCode: statusLine.statusCode,
			httpVersion: statusLine.httpVersion
		};
	};

	this._parseHeaders = function(headers){
		var newHeaders = {}, pos = 0;

		for(var i = 0, l = headers.length; i < l; i++){
			//wiem ze ten kod jest glupi ale split w javascripcie jest jeszcze glupszy
			pos = headers[i].indexOf(':');
			newHeaders[headers[i].substring(0, pos).trim()] = headers[i].substring(pos + 1, headers[i].length).trim();
		}

		return newHeaders;
	};

	this._headAsString = function(){
		return this._statusLineAsString() + "\r\n" + this._headersAsString() + "\r\n";
	};

	this._headersAsString = function(){
		var str = '';

		for(var key in this.getHeaders()){
			if(this._headers.hasOwnProperty(key)){
				str += key + ':' + this._headers[key].join(',') + "\r\n";
			}
		}

		return str;
	};

	this.toBuffer = function(){
		this.setHeader('content-length', this.getBody().length(), true);

		var headBuffer = new Buffer(this._headAsString());
		var bodyBuffer = this.getBody().toBuffer();

		var buffer = new Buffer(headBuffer.length + bodyBuffer.length);
		
		headBuffer.copy(buffer);
		bodyBuffer.copy(buffer, headBuffer.length);

		return buffer;
	};

	this.toBinaryData = function(){
		return new BinaryData(this.toBuffer(), this.getBody().getEncoding(), this.getBody().getCharacterEncoding());
	};

    /**
     * sets request headers
     * could be Object with key/value pais or
     * two String Objects first with a header name, second with the header value
     *
     * @param {String} headerName
     * @param {String} headerValue
     * @param {Object} headers
     * @returns {UrlRequest}
     */
    this.setHeaders = function(headers, overwrite){
        overwrite = overwrite || false;
        for(var key in headers){
			if(headers.hasOwnProperty(key)){
				this.setHeader(key, headers[key], overwrite);
			}
		}

        return this;
    };

	this.setHeader = function(name, value, overwrite){
		var headerName = name.toLowerCase(), headerValue, overwrite = overwrite || false;

		if(headerName == "host" || headerName == "cookie" || headerName == "set-cookie"){
			this._headers[headerName] = value;
		}else{
			if(this._headers.hasOwnProperty(name) && !overwrite){
				if(typeof value == "object" && value.constructor == Array){
					headerValue = this._headers[name].concat(value);
				}else{
					headerValue = this._headers[name].concat([value])
				}
			}else{
				if(typeof value == "object" && value.constructor == Array){
					headerValue = value
				}else{
					headerValue = [value];
				}
			}

			this._headers[headerName] = headerValue;
		}

		return this;
	};

	this.removeHeader = function(name){
		var headerName = name.toLowerCase();
		if(this._headers.hasOwnProperty(headerName)){
			delete this._headers[headerName];

			return true;
		}

		return false;
	};

    /**
     * returns headers value pair object
     * @returns {Object}
     */
    this.getHeaders = function(){
        return this._headers;
    };

    this.getHeadersString = function(){
        return this._headers.join("\r\n");
    };

	this.getHeader = function(name){
		var headerName = name.toLowerCase();

		if(this._headers.hasOwnProperty(headerName)){
			if(this._headers[name].constructor == Array && this._headers[name].length == 1){
				return this._headers[name][0];
			}else{
				return this._headers[name];
			}
		}

		return null;
	}

    /**
     * sets the body
     * @param {Object} body
     * @returns {Object}
     */
    this.setBody = function(body){
		this._body = body;

        return this;            
    };

    /**
     * returns body
     * @returns {Object}
     */
    this.getBody = function(){
        return this._body;
    };

	this.setHttpVersion = function(httpVersion){
		this._httpVersion = httpVersion;

		return this;
	};

	this.getHttpVersion = function(){
		return this._httpVersion;
	};

	this.setCookie = function(name, value){
		if(!this._cookieKey){
			return false;
		}

		var cookies = this.getHeader(this._cookieKey);

		if(cookies == null){
			cookies = {};
		}

		cookies[name] = value;

		this.setHeader(this._cookieKey, cookies);

		return this;
	};

	this.getCookie = function(name){
		if(!this._cookieKey){
			return false;
		}

		var cookies = this.getHeader(this._cookieKey);

		if(cookies == null){
			return null;
		}

		return (cookies.hasOwnProperty(name)) ? cookies[name] : null;
	};

	this.removeCookie = function(name, value){
		if(!this._cookieKey){
			return false;
		}

		var cookies = this.getHeader(this._cookieKey);

		if(cookies == null){
			return this;
		}

		if(cookies.hasOwnProperty(name)){
			delete cookies[name];
		}

		this.setHeader(this._cookieKey, cookies);

		return this;
	};
};

Message = new Class(new Message());

Message.Exception = {};
Message.Exception.WRONG_BUFFER = "Buffer doesn't contain full http response";

exports.Message = Message;
