var Message = require('./Message.js').Message;
var Class = require('../class/Class.js').Class;

var assertions = require('../common/Assertions.js');
var types = require('../common/Types.js');
var BinaryData = require('../data/BinaryData.js').BinaryData;

var urlParser = require('url');
var http = require('http');
var util = require('util');

/**
 * @class Request
 * @requires Class | Message
 *
 * @param {String} url
 */
var Request = function(){

	this.Extends = Message;

    /**
     * @private
     * @property {String} _url
     */
    this._url = null;

    /**
     * @private
     * @property {String} _method
     */
    this._method = 'GET';

	this._host = null;

	this._port = 80;

	this._path = '/';

	this._proto = 'http:';

	this._authUser = null;

	this._authPassword = null;

	this._query = null;

	this._cookieKey = 'cookie';
	
	this.initialize = function(params){
		this.parent(params);

		if(types.isString(params)){
			this.setUrl(params);
		}else if(types.isInstanceOf(params, Request)){
			this.setUrl(params.getUrl());
			this.setMethod(params.getMethod());
			this.setHeaders(params.getHeaders());
			this.setBody(params.getBody());
			this.setHttpVersion(params.getHttpVersion());
		}else if(types.isInstanceOf(params, http.IncomingMessage)){
			this.setUrl('http://' + params.headers.host + params.url);
			this.setMethod(params.method);
			this.setHeaders(params.headers);
			this.setHttpVersion(params.httpVersion);
            this.setBody(params.data);
		}
	}


    /**
     * sets url of a service endpoint
     *
     * @param {String} url
     * @returns {UrlRequest}
     */
    this.setUrl = function(url){
        this._url = url;

		var matches = urlParser.parse(url, true, false);

		if(matches.hasOwnProperty('protocol')){
			this.setProto(matches.protocol);
		}

		if(matches.hasOwnProperty('auth')){
			this.setAuth(matches.auth);
		}

		if(matches.hasOwnProperty('hostname')){
			this.setHost(matches.hostname);
		}

		if(matches.hasOwnProperty('port')){
			this.setPort(matches.port);
		}

		if(matches.hasOwnProperty('pathname')){
			this.setPath(matches.pathname);
		}

		if(matches.hasOwnProperty('query')){
			this.setQuery(matches.query);
		}

        return this;
    };

    /**
     * returns url of a service endpoint
     *
     * @returns {String}
     */
    this.getUrl = function(){
        var url = '';

		var proto = this.getProto();
		if(proto){
			url += proto + '//';
		}

		var auth = this.getAuth();
		if(auth){
			url += auth + '@';
		}

		var host = this.getHost(), port = this.getPort();
		if(this.getPort()){
			url += host + ':' + port;
		}else{
			url += host;
		}

		var path = this.getPath();
		if(path){
			url += path;
		}else{
			url += '/';
		}

		var query = this.getQuery()
		if(query){
			url += this.getQueryString();
		}

		return url;
    };

    this.getQueryString = function(){
        var url = '';
        var query = this.getQuery()
		if(query){
			url += '?';

			var tmp = '';

			for(var key in query){
				if(query.hasOwnProperty(key)){
					tmp += encodeURIComponent(key) + '=' + encodeURIComponent(query[key]) + '&';
				}
			}

			url += tmp.substring(0, tmp.length - 1);
		}
        if(url==='?'){
            url = '';
        }
        return url;
    };


	//to bylo po cos potrzebne ale nie pamietam po co
	/*this.setHeader = function(name, value){
		this.parent(name, value);

		if(name.toLowerCase() == 'host'){
			this.setHost(value);
		}
	};*/

    /**
     * sets method of communicating with a service
     *
     * @param {String} method
     * @returns {UrlRequest}
     */
    this.setMethod = function(method){
        this._method = method;

        return this;
    };

    /**
     * returns method of communicating with a service
     *
     * @returns {String}
     */
    this.getMethod = function(){
        return this._method;
    };

	this.setHost = function(host){
		this._host = host;

		return this;
	};

	this.getHost = function(){
		return this._host;
	};

	this.setPort = function(port){
		this._port = port;

		return this;
	};

	this.getPort = function(){
		return this._port;
	};

	this.setPath = function(path){
		this._path = path;

		return this;
	};

	this.getPath = function(){
		return this._path;
	};

	this.setProto = function(proto){
		this._proto = proto;

		return this;
	};

	this.getProto = function(){
		return this._proto;
	};

	this.setAuth = function(auth){
		var data = auth.split(':');

		this.setAuthUser(data[0]);
		this.setAuthPassword(data[1]);

		return this;
	};

	this.getAuth = function(){
		var user = this.getAuthUser(), password = this.getAuthPassword();

		if(user && password){
			return user + ':' + password;
		};

		return null;
	};

	this.setAuthUser = function(user){
		this._authUser = user;

		return this;
	};

	this.getAuthUser = function(){
		return this._authUser;
	};

	this.setAuthPassword = function(password){
		this._authPassword = password;

		return this;
	};

	this.getAuthPassword = function(){
		return this._authPassword;
	};

	this.setQuery = function(query){
		this._query = query;
	};

	this.getQuery = function(){
		return this._query;
	};

	this.setQueryParam = function(name, value){
		if(!this._query){
			this._query = {};
		};

		this._query[name] = value;

		return this;
	};

	this.getQueryParam = function(name, value){
		if(!this._query){
			return null;
		}

		return (this._query.hasOwnProperty(name)) ? this._query[name] : null;
	};

	this.setBody = function(body){
        if(types.isInstanceOf(body, BinaryData)){
            this.parent(body);
        }else{
            this.parent(new BinaryData(body, BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8));
        }
	};
    
    this.isCachingDisabled = function(){
        var cachingDisabled = false;
        cachingDisabled = (this.getMethod() == Request.POST);
        cachingDisabled = cachingDisabled || this.getHeader('pragma') == 'no-cache';
        cachingDisabled = cachingDisabled || this.getHeader('cache-control') == 'no-cache';
        return cachingDisabled;
    }
    
    this.isConditional = function(){
        return this.getHeader('if-modified-since');
    }
};

Request = new Class(new Request());
Request.OPTIONS = "OPTIONS";
Request.GET = "GET";
Request.HEAD = "HEAD";
Request.POST = "POST";
Request.PUT = "PUT";
Request.DELETE = "DELETE";
Request.TRACE = "TRACE";
Request.CONNECT = "CONNECT";

Request.Exception = {};
Request.Exception.WRONG_BODY = "Body has to be a instance of BinaryData";

exports.Request = Request;
